(* Winmerge Plugin
   ---------------
   SUnpack gz compressed file

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a).rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   created: 2011-09-20
   last modified: 2018-03-20
   *)

unit PbUnpackUnit;

interface

uses
  Winapi.Windows, System.Win.ComObj;

// *********************************************************************//
// GUIDS declared in the TypeLibrary. Following prefixes are used:      //
//   Type Libraries     : LIBID_xxxx                                    //
//   CoClasses          : CLASS_xxxx                                    //
//   DISPInterfaces     : DIID_xxxx                                     //
//   Non-DISP interfaces: IID_xxxx                                      //
// *********************************************************************//
const
  LIBID_PbUnpackLib: TGUID = '{E1EE0F86-298A-46DD-945D-517AFDD0718C}';
  IID_IWinMergeScript: TGUID = '{3E669C5B-0642-4D45-A6D2-B8B7972B45DD}';
  CLASS_WinMergeScript: TGUID = '{A0AA7EDF-D2FC-4237-8CBF-CD0EA03C9816}';
type

// *********************************************************************//
// Forward declaration of interfaces defined in Type Library            //
// *********************************************************************//
  IWinMergeScript = interface;
  IWinMergeScriptDisp = dispinterface;

// *********************************************************************//
// Declaration of CoClasses defined in Type Library                     //
// (NOTE: Here we map each CoClass to its Default Interface)            //
// *********************************************************************//
  WinMergeScript = IWinMergeScript;

// *********************************************************************//
// Interface: IWinMergeScript
// Flags:     (4416) Dual OleAutomation Dispatchable
// *********************************************************************//
  IWinMergeScript = interface(IDispatch)
    ['{3E669C5B-0642-4D45-A6D2-B8B7972B45DD}']
    function Get_PluginEvent: WideString; safecall;
    function Get_PluginDescription: WideString; safecall;
    function Get_PluginFileFilters: WideString; safecall;
    function Get_PluginIsAutomatic: WordBool; safecall;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
    property PluginEvent: WideString read Get_PluginEvent;
    property PluginDescription: WideString read Get_PluginDescription;
    property PluginFileFilters: WideString read Get_PluginFileFilters;
    property PluginIsAutomatic: WordBool read Get_PluginIsAutomatic;
  end;

// *********************************************************************//
// DispIntf:  IWinMergeScriptDisp
// Flags:     (4416) Dual OleAutomation Dispatchable
// GUID:      {DA4E2506-F060-11D7-9350-444553540000}
// *********************************************************************//
  IWinMergeScriptDisp = dispinterface
    ['{3E669C5B-0642-4D45-A6D2-B8B7972B45DD}']
    property PluginEvent: WideString readonly dispid 1;
    property PluginDescription: WideString readonly dispid 2;
    property PluginFileFilters: WideString readonly dispid 3;
    property PluginIsAutomatic: WordBool readonly dispid 4;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; dispid 7;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; dispid 8;
  end;

  CoWinMergeScript = class
    class function Create: IWinMergeScript;
    class function CreateRemote(const MachineName: string): IWinMergeScript;
  end;

  TWinMergeScript = class(TAutoObject, IWinMergeScript)
  protected
    function Get_PluginDescription : WideString; safecall;
    function Get_PluginEvent : WideString; safecall;
    function Get_PluginFileFilters : WideString; safecall;
    function Get_PluginIsAutomatic : WordBool; safecall;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
  end;

implementation

uses
  System.Win.ComServ, System.SysUtils, System.Classes, System.StrUtils,
  Vcl.Forms, Vcl.Controls, WinUtils, FileUtils, FileConsts, ExtFileTools,
  AesLib, PwdDlg;

{ ------------------------------------------------------------------- }
function TWinMergeScript.Get_PluginDescription : WideString;
begin
  result := 'Personal Backup unpack plugin';
end;

function TWinMergeScript.Get_PluginEvent : WideString;
begin
  result := 'FILE_PACK_UNPACK';
end;

function TWinMergeScript.Get_PluginFileFilters : WideString;
begin
  result := '\.gz$;\.gze$;\.enc$';
end;

function TWinMergeScript.Get_PluginIsAutomatic : WordBool;
begin
  result := true;
end;

{ ------------------------------------------------------------------- }
// unpack to file
function TWinMergeScript.UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
var
  GzInfo      : TGzFileInfo;
  Decryption  : TEncryption;
  CryptPwd    : AnsiString;
  n           : integer;
  EMsg        : string;
begin
  pChanged := false;
  result := false;
//  InfoDialog('Write to temp file: '+filedst);
  if not (AnsiSameText(GetExt(fileSrc),'gz') or AnsiSameText(GetExt(fileSrc),'gze'))then begin  // no unpacking
    if not AnsiSameText(GetExt(fileSrc),'enc') then begin  // not encrypted
      with TWinCopyThread.Create(fileSrc,fileDst,nil,[],true,defBufferSize,tpNormal) do begin
        Overwrite:=true;
        Resume;
        repeat
          Sleep(10);
          Application.ProcessMessages;
          until Done;
        EMsg:=SysErrorMsg;
        n:=ErrorCode;
        Free;
        end;
      end
    else begin
      CryptPwd:='';
      if ReadPassword('Personal Backup Unpack',Format('To read encrypted file:'+sLineBreak+'%s'+sLineBreak+'requires a password!',
          [StripPath(fileSrc,50)]),false,false,CryptPwd,n)=mrOK then begin
        Decryption:=TEncryption.Create(CryptPwd,0,defBufferSize);
        end
      else Exit;
      with TDecryptThread.Create(fileSrc,fileDst,Decryption,true,defBufferSize,tpNormal) do begin
        Overwrite:=true;
        Resume;
        repeat
          Sleep(10);
          Application.ProcessMessages;
          until Done;
        EMsg:=SysErrorMsg;
        n:=ErrorCode;
        Free;
        end;
      FreeAndNil(Decryption);
      end;
    end
  else begin
    if not GzFileInfoXL(fileSrc,GzInfo) then Exit;
    if GzInfo.Encrypted then begin
      CryptPwd:='';
      if ReadPassword('Personal Backup Unpack',Format('To unpack encrypted file:'+sLineBreak+'%s'+sLineBreak+'requires a password!',
          [StripPath(fileSrc,50)]),false,false,CryptPwd,n)=mrOK then begin
        Decryption:=TEncryption.Create(CryptPwd,0,defBufferSize);
        end
      else Exit;
      end
    else begin
      Decryption:=nil;
      end;
    with TGUnZipThread.Create(fileSrc,fileDst,Decryption,true,defBufferSize,tpNormal) do begin
      Overwrite:=true;
      Resume;
      repeat
        Sleep(10);
        Application.ProcessMessages;
        until Done;
      EMsg:=SysErrorMsg;
      n:=ErrorCode;
      Free;
      end;
    if assigned(Decryption) then FreeAndNil(Decryption);
    end;
  if n=errOK then begin
    pChanged := true;
    result := true;
    end
  else ErrorDialog(GetCopyErrMsg(n)+sLineBreak+EMsg);
  end;

{ ------------------------------------------------------------------- }
// no packing supported
function TWinMergeScript.PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
begin
  pChanged := false;
  Result := false;
  end;

class function CoWinMergeScript.Create: IWinMergeScript;
begin
  Result:=CreateComObject(CLASS_WinMergeScript) as IWinMergeScript;
end;

class function CoWinMergeScript.CreateRemote(const MachineName: string): IWinMergeScript;
begin
  Result:=CreateRemoteComObject(MachineName, CLASS_WinMergeScript) as IWinMergeScript;
end;


initialization
  TAutoObjectFactory.Create(ComServer, TWinMergeScript, Class_WinMergeScript,
    ciMultiInstance, tmApartment);
end.

