(* Winmerge Plugin
   ---------------
   example for plugin
   

    J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   created: 2011-08-01
   *)

unit MyPluginUnit;

interface

uses
  Windows, ComObj;

// *********************************************************************//
// GUIDS declared in the TypeLibrary. Following prefixes are used:      //
//   Type Libraries     : LIBID_xxxx                                    //
//   CoClasses          : CLASS_xxxx                                    //
//   DISPInterfaces     : DIID_xxxx                                     //
//   Non-DISP interfaces: IID_xxxx                                      //
// *********************************************************************//
const
  LIBID_MyPluginLib: TGUID = '{5E7787FD-4A98-4113-B48E-04D7C8D6C035}';
  IID_IWinMergeScript: TGUID = '{7D1D732C-423A-4FFE-AA6B-60222691FA9E}';
  CLASS_WinMergeScript: TGUID = '{0D7B39BF-2540-422F-9BB2-A8DFBA7C15EE}';
type

// *********************************************************************//
// Forward declaration of interfaces defined in Type Library            //
// *********************************************************************//
  IWinMergeScript = interface;
  IWinMergeScriptDisp = dispinterface;

// *********************************************************************//
// Declaration of CoClasses defined in Type Library                     //
// (NOTE: Here we map each CoClass to its Default Interface)            //
// *********************************************************************//
  WinMergeScript = IWinMergeScript;

// *********************************************************************//
// Interface: IWinMergeScript
// Flags:     (4416) Dual OleAutomation Dispatchable
// *********************************************************************//
  IWinMergeScript = interface(IDispatch)
    ['{7D1D732C-423A-4FFE-AA6B-60222691FA9E}']
    function Get_PluginEvent: WideString; safecall;
    function Get_PluginDescription: WideString; safecall;
    function Get_PluginFileFilters: WideString; safecall;
    function Get_PluginIsAutomatic: WordBool; safecall;
    function PrediffBufferW(var pText : WideString; var pSize : Integer;
                        var pChanged: WordBool): WordBool; safecall;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
    property PluginEvent: WideString read Get_PluginEvent;
    property PluginDescription: WideString read Get_PluginDescription;
    property PluginFileFilters: WideString read Get_PluginFileFilters;
    property PluginIsAutomatic: WordBool read Get_PluginIsAutomatic;
  end;

// *********************************************************************//
// DispIntf:  IWinMergeScriptDisp
// Flags:     (4416) Dual OleAutomation Dispatchable
// GUID:      {DA4E2506-F060-11D7-9350-444553540000}
// *********************************************************************//
  IWinMergeScriptDisp = dispinterface
    ['{7D1D732C-423A-4FFE-AA6B-60222691FA9E}']
    property PluginEvent: WideString readonly dispid 1;
    property PluginDescription: WideString readonly dispid 2;
    property PluginFileFilters: WideString readonly dispid 3;
    property PluginIsAutomatic: WordBool readonly dispid 4;
    function PrediffBufferW(var pText : WideString; var pSize : Integer;
                        var pChanged: WordBool): WordBool; dispid 5;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; dispid 7;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; dispid 8;
  end;

  CoWinMergeScript = class
    class function Create: IWinMergeScript;
    class function CreateRemote(const MachineName: string): IWinMergeScript;
  end;

  TWinMergeScript = class(TAutoObject, IWinMergeScript)
  protected
    function Get_PluginDescription : WideString; safecall;
    function Get_PluginEvent : WideString; safecall;
    function Get_PluginFileFilters : WideString; safecall;
    function Get_PluginIsAutomatic : WordBool; safecall;
    function PrediffBufferW(var pText : WideString; var pSize : Integer;
                        var pChanged: WordBool): WordBool; safecall;
    function UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
    function PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
  end;

type
  TStringValue = class(TObject)
    FString : string;
    constructor Create (const AString : string);
    end;

implementation

uses
  ComServ, SysUtils, Classes, StrUtils;

function WriteTextFile (const FName,Text : string) : boolean;
var
  sa : AnsiString;
  fs : TFileStream;
begin
  sa:=Text; Result:=false;
  fs:= TFileStream.Create(FName,fmCreate);
  try
    fs.Write(sa[1],length(sa));
    Result:=true;
  finally
    fs.Free;
    end;
  end;

{ ------------------------------------------------------------------- }
function TWinMergeScript.Get_PluginDescription : WideString;
begin
  result := 'My plugin';
end;

function TWinMergeScript.Get_PluginEvent : WideString;
begin
  result := 'FILE_PACK_UNPACK';  // or 'BUFFER_PREDIFF'
end;

function TWinMergeScript.Get_PluginFileFilters : WideString;
begin
  result := '\.xyz$;\.abc$';    // filter for automatic use
end;

function TWinMergeScript.Get_PluginIsAutomatic : WordBool;
begin
  result := true;
end;

{ ------------------------------------------------------------------- }
// remove leading line numbers
function TWinMergeScript.PrediffBufferW(var pText : WideString; var pSize : Integer;
                        var pChanged: WordBool): WordBool; safecall;
begin
  Result:=true;
  end;

{ ------------------------------------------------------------------- }
// unpack to file
function TWinMergeScript.UnpackFile(const fileSrc: WideString; const fileDst: WideString;
                        var pChanged: WordBool; var pSubcode: Integer): WordBool; safecall;
var
  src            : TFileStream;
  sText          : string;
  sa             : AnsiString;
  c              : Char;
begin
  try
    src := TFileStream.Create(fileSrc,fmOpenRead);
    with src do begin
      Read(c,2);
      if c=#$FeFF then begin  // Unicode
        SetLength(sText,Size);
        Read(s[1],2*(Size-1));
        sText[Size]:=#0;
        end
      else begin // Iso
        Seek(0,soFromBeginning);
        SetLength(sa,Size+1);
        Read(sa[1],Size);
        sa[Size]:=#0;
        sText:=sa;
        end;
      Free;
      end;
  // sText contains the text from file "filesrc"
  // change it as desired for unpacking
    pChanged:=WriteTextFile(fileDst,sText);
    Result:=pChanged;
  except
    pChanged := false;
    Result := false;
    end;
  end;

{ ------------------------------------------------------------------- }
// pack file
function TWinMergeScript.PackFile(const fileSrc: WideString; const fileDst: WideString; var pChanged: WordBool;
                      Subcode: Integer): WordBool; safecall;
var
  src            : TFileStream;
  sText          : string;
  sa             : AnsiString;
begin
  try
    src := TFileStream.Create(fileSrc,fmOpenRead);
    with src do begin          // read merged text
      SetLength(sa,Size+1);
      Read(sa[1],Size);
      sa[Size]:=#0;
      Free;
      end;
    sText:=sa;
  // sText contains the text from file "filesrc"
  // change it as desired for packing
    pChanged:=WriteTextFile(fileDst,sText);
    Result:=pChanged;
  except
    pChanged := false;
    Result := false;
    end;
  end;

class function CoWinMergeScript.Create: IWinMergeScript;
begin
  Result := CreateComObject(CLASS_WinMergeScript) as IWinMergeScript;
end;

class function CoWinMergeScript.CreateRemote(const MachineName: string): IWinMergeScript;
begin
  Result := CreateRemoteComObject(MachineName, CLASS_WinMergeScript) as IWinMergeScript;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TWinMergeScript, Class_WinMergeScript,
    ciMultiInstance, tmApartment);
end.

